@@ -136,6 +136,35 @@ void init_page(py::module_ &m)
                 arbitrary spots, such as in the middle of a token, as that can
                 confuse some software.
             )~~~")
+        .def(
+            "contents_add",
+            [](QPDFPageObjectHelper &poh, QPDFObjectHandle &contents, bool prepend) {
+                return poh.addPageContents(contents, prepend);
+            },
+            py::arg("contents"),
+            // py::kw_only(), // TODO in 3.x
+            py::arg("prepend") = false,
+            py::keep_alive<1, 2>(),
+            R"~~~(
+                Append or prepend to an existing page's content stream using an existing stream object.
+            )~~~")
+        .def(
+            "contents_add",
+            [](QPDFPageObjectHelper &poh, py::bytes contents, bool prepend) {
+                auto q = poh.getObjectHandle().getOwningQPDF();
+                if (!q) {
+                    // LCOV_EXCL_LINE
+                    throw std::logic_error("QPDFPageObjectHelper not attached to QPDF");
+                }
+                auto stream = QPDFObjectHandle::newStream(q, contents);
+                return poh.addPageContents(stream, prepend);
+            },
+            py::arg("contents"),
+            py::kw_only(),
+            py::arg("prepend") = false,
+            R"~~~(
+                Append or prepend to an existing page's content stream from bytes.
+            )~~~")
         .def("remove_unreferenced_resources",
             &QPDFPageObjectHelper::removeUnreferencedResources,
             R"~~~(
@@ -171,6 +200,51 @@ void init_page(py::module_ &m)
                         dictionary. In this way, the page's transformations will
                         be preserved when placing this object on another page.
             )~~~")
+        .def(
+            "calc_form_xobject_placement",
+            [](QPDFPageObjectHelper &poh,
+                QPDFObjectHandle formx,
+                QPDFObjectHandle name,
+                QPDFObjectHandle::Rectangle rect,
+                bool invert_transformations,
+                bool allow_shrink,
+                bool allow_expand) -> py::bytes {
+                return poh.placeFormXObject(formx,
+                    name.getName(),
+                    rect,
+                    invert_transformations,
+                    allow_shrink,
+                    allow_expand);
+            },
+            py::arg("formx"),
+            py::arg("name"),
+            py::arg("rect"),
+            py::kw_only(),
+            py::arg("invert_transformations") = true,
+            py::arg("allow_shrink")           = true,
+            py::arg("allow_expand")           = false,
+            R"~~~(
+                Generate content stream segment to place a Form XObject on this page.
+
+                The content stream segment must be then be added to the page's
+                content stream.
+
+                The default keyword parameters will preserve the aspect ratio.
+
+                Args:
+                    formx: The Form XObject to place.
+                    name: The name of the Form XObject in this page's /Resources
+                        dictionary.
+                    rect: Rectangle describing the desired placement of the Form
+                        XObject.
+                    invert_transformations: Apply /Rotate and /UserUnit scaling
+                        when determining FormX Object placement.
+                    allow_shrink: Allow the Form XObject to take less than the
+                        full dimensions of rect.
+                    allow_expand: Expand the Form XObject to occupy all of rect.
+
+                .. versionadded:: 2.14
+            )~~~")
         .def(
             "get_filtered_contents",
             [](QPDFPageObjectHelper &poh, TokenFilter &tf) {
@@ -260,7 +334,7 @@ void init_page(py::module_ &m)
 
                 Requires O(n) search.
 
-                .. versionadded: 2.2
+                .. versionadded:: 2.2
             )~~~")
         .def_property_readonly(
             "label",
@@ -294,9 +368,9 @@ void init_page(py::module_ &m)
                 Note that this requires a O(n) search over all pages, to look up
                 the page's index.
 
-                .. versionadded: 2.2
+                .. versionadded:: 2.2
 
-                .. versionchanged: 2.9
+                .. versionchanged:: 2.9
                     Returns the ordinary page number if no special rules for page
                     numbers are defined.
             )~~~");
@@ -349,10 +423,7 @@ void init_page(py::module_ &m)
                     bytes
             )~~~")
         .def_property_readonly("error_msg", &QPDFTokenizer::Token::getErrorMessage)
-        .def("__eq__", &QPDFTokenizer::Token::operator==)
-        .def("__eq__", [](const QPDFTokenizer::Token &t, py::object other) {
-            return py::reinterpret_borrow<py::object>(py::handle(Py_NotImplemented));
-        });
+        .def("__eq__", &QPDFTokenizer::Token::operator==, py::is_operator());
 
     py::class_<QPDFObjectHandle::TokenFilter,
         PointerHolder<QPDFObjectHandle::TokenFilter>>
