@@ -1,20 +1,23 @@
 import pytest
 
-import pikepdf
+from pikepdf import Array, Dictionary, Name, Page, Pdf, Rectangle
 
-Array = pikepdf.Array
-Dictionary = pikepdf.Dictionary
-Name = pikepdf.Name
+# pylint: disable=redefined-outer-name
 
 
 @pytest.fixture
 def graph(resources):
-    return pikepdf.open(resources / 'graph.pdf')
+    yield Pdf.open(resources / 'graph.pdf')
+
+
+@pytest.fixture
+def fourpages(resources):
+    yield Pdf.open(resources / 'fourpages.pdf')
 
 
 @pytest.fixture
 def graph_page(graph):
-    return pikepdf.Page(graph.pages[0])
+    return Page(graph.pages[0])
 
 
 def test_page_boxes(graph_page):
@@ -32,7 +35,7 @@ def test_page_boxes(graph_page):
     assert page.mediabox != page.cropbox
     assert page.cropbox != page.mediabox
 
-    page.cropbox = pikepdf.Array([0, 0, 50, 50])
+    page.cropbox = Array([0, 0, 50, 50])
 
 
 def test_invalid_boxes(graph_page):
@@ -46,7 +49,9 @@ def test_invalid_boxes(graph_page):
 
 
 def test_page_repr(graph_page):
-    assert repr(graph_page).startswith('<pikepdf.Page')
+    r = repr(graph_page)
+    assert r.startswith('<pikepdf.Page')
+    assert '(Type="/Page")' not in r
 
 
 class TestAddResource:
@@ -89,14 +94,14 @@ class TestAddResource:
 
 
 def test_add_unowned_page():  # issue 174
-    pdf = pikepdf.new()
-    d = pikepdf.Dictionary(Type=pikepdf.Name.Page)
+    pdf = Pdf.new()
+    d = Dictionary(Type=Name.Page)
     pdf.pages.append(d)
 
 
 def test_failed_add_page_cleanup():
-    pdf = pikepdf.new()
-    d = pikepdf.Dictionary(Type=pikepdf.Name.NotAPage)
+    pdf = Pdf.new()
+    d = Dictionary(Type=Name.NotAPage)
     num_objects = len(pdf.objects)
     with pytest.raises(TypeError, match="only pages can be inserted"):
         pdf.pages.append(d)
@@ -108,9 +113,44 @@ def test_failed_add_page_cleanup():
     assert pdf.objects[-1] is None, "Left a stale object behind without deleting"
 
     # But we'd better not delete an existing object...
-    d2 = pdf.make_indirect(pikepdf.Dictionary(Type=pikepdf.Name.StillNotAPage))
+    d2 = pdf.make_indirect(Dictionary(Type=Name.StillNotAPage))
     with pytest.raises(TypeError, match="only pages can be inserted"):
         pdf.pages.append(d2)
     assert len(pdf.pages) == 0
 
     assert d2.same_owner_as(pdf.Root)
+
+
+def test_formx(graph, outpdf):
+    formx = Page(graph.pages[0]).as_form_xobject()
+    graph.add_blank_page()
+    new_page = Page(graph.pages[-1])
+    formx_placed_name = new_page.add_resource(formx, Name.XObject)
+    cs = new_page.calc_form_xobject_placement(
+        formx, formx_placed_name, Rectangle(0, 0, 200, 200)
+    )
+    assert bytes(formx_placed_name) in cs
+    new_page.obj.Contents = graph.make_stream(cs)
+    graph.save(outpdf)
+
+
+def test_fourpages_to_4up(fourpages, graph, outpdf):
+    pdf = Pdf.new()
+    pdf.add_blank_page(page_size=(1000, 1000))
+    page = Page(pdf.pages[0])
+
+    pdf.pages.extend(fourpages.pages)
+
+    page.add_overlay(pdf.pages[1], Rectangle(0, 500, 500, 1000))
+    page.add_overlay(Page(pdf.pages[2]), Rectangle(500, 500, 1000, 1000))
+    page.add_overlay(Page(pdf.pages[3]).as_form_xobject(), Rectangle(0, 0, 500, 500))
+    page.add_underlay(pdf.pages[4], Rectangle(500, 0, 1000, 500))
+
+    page.add_underlay(graph.pages[0])
+
+    with pytest.raises(TypeError):
+        page.add_overlay(Dictionary(Key=123))
+
+    del pdf.pages[1:]
+
+    pdf.save(outpdf)
